require 'sinatra'
require 'fileutils'
require 'erb'

if File.exist?("config.rb") 
  require "config.rb"
else
  set :environment, :development
  set :auth_save, true
	set :key_length, 1024
  #change this setting to install several instances in one vhost with passenger
  set :base, ""
end


helpers do 
  def clean_filename(s)
    s.sub(/\.aes$/,"")
  end
end

if settings.auth_save
  require 'openssl'
  require 'base64'
  #Load saved keys but generate new ones if not found or if world readable
  if File.exist?("keys.pem") and ! File.world_readable?("keys.pem")
		KEY = OpenSSL::PKey::RSA.new(File.read("keys.pem"))
  else
		KEY = OpenSSL::PKey::RSA.new(settings.key_length)
		pub = KEY.public_key.to_pem
		priv = KEY.to_pem
		File.open("keys.pem", "w+") { |f| f.write(pub+priv) }
		File.chmod(0600, "keys.pem")
  end
end

get '/' do
	@title = "List of files"
  @list = []
  Dir.foreach("safe") do |f|
    next unless /.*\.aes$/.match(f) 
    @list.push clean_filename(f)
  end
  erb :index
end

get '/view/:file' do
  file = "safe/#{params[:file]}.aes"
  @file_id = params[:file]
  @title = clean_filename(@file_id)
  if File.exists?(file)
    @encrypted_content = File.read(file)
    erb :view
  else
    error 404, "The file you requested was not found"
  end
end

get '/new' do
	@title="File creation"
  erb :new
end
  

post '/create' do

  # refuse to create existing files  
  if File.exists?(params[:file]) or /\.\./.match(params[:file])
    error 500, "Error"
  else
    # save md5 sent if needed
		if settings.auth_save
			md5 = KEY.private_decrypt(Base64.decode64(params[:md5]))
			File.open("safe/#{params[:file]}.md5", 'w') {|f| f.write(md5) }
		end
		#save encrypted content
		File.open("safe/#{params[:file]}.aes", 'w') {|f| f.write(params[:encrypted]) }
  end
  [ 200, "Created" ]

end

post "/update" do
  if ( !File.exists?("safe/#{params[:file]}.aes") ) 
    error 409, "File exists: #{params[:file]}"
	end
	if  /\.\./.match(params[:file])
    error 403, "Invalid name: #{params[:file]}"
	end

	# check md5 sent if needed and if md5 present (eases the transition to enabling md5 checking)
  if settings.auth_save and File.exist?("safe/#{params[:file]}.md5")
		#FIXME : should handle an error here: would mean the server keys were changed between the read and the save by the client
		md5_str = KEY.private_decrypt(Base64.decode64(params[:md5]))
		new_md5, old_md5 = md5_str.split(/---/)
		local_old_md5 = File.open("safe/#{params[:file]}.md5", "rb").read
		if old_md5!=local_old_md5
			return [ 409, "File Changed During edition"]
		end
	end
		
	FileUtils.cp("safe/#{params[:file]}.aes", "backups/#{params[:file]}.#{Time.now.strftime("%Y-%m-%d:%H:%M:%S")}.aes")
	if settings.auth_save
		File.open("safe/#{params[:file]}.md5", 'w') {|f| f.write(new_md5) }
	end
	File.open("safe/#{params[:file]}.aes", 'w') {|f| f.write(params[:encrypted]) }
	[ 200, "Updated" ]
end

get "/download/:file" do
  file = "safe/#{params[:file]}.aes"
  send_file(file, :disposition =>'attachment', :filename => params[:file])
end
